package dev.peerat.parser.java;

import java.util.ArrayList;
import java.util.List;
import java.util.function.Function;

import dev.peerat.parser.Bag;
import dev.peerat.parser.Token;
import dev.peerat.parser.java.Annotation.AnnotableBuffer;
import dev.peerat.parser.java.visitor.JavaVisitor;
import dev.peerat.parser.visitor.VisitorBag;

public class JavaFile extends JavaElement implements ClassContainer, AnnotableBuffer{
	
	private List<Annotation> annotationBuffer;
	
	private Token pack;
	private List<Import> imports;
	private ClassBase mainClazz;
	private List<ClassBase> subClazz;
	
	public JavaFile(){
		this.imports = new ArrayList<>();
		this.subClazz = new ArrayList<>();
	}
	
	JavaFile setPackage(Bag bag){
		this.pack = bag.<Token>get();
		return this;
	}
	
	JavaFile addImport(Import value){
		this.imports.add(value);
		return this;
	}
	
	@Override
	public void addClass(ClassBase clazz){
		if(mainClazz == null) this.mainClazz = clazz;
		else subClazz.add(clazz);
	}
	
	public Token getPackage(){
		return this.pack;
	}
	
	public List<Import> getImports(){
		return this.imports;
	}
	
	public ClassBase getMainClass(){
		return this.mainClazz;
	}
	
	public List<ClassBase> getSubClasses(){
		return this.subClazz;
	}
	
	@Override
	public void addAnnotationBuffer(Annotation annotation){
		if(annotationBuffer == null) annotationBuffer = new ArrayList<>();
		annotationBuffer.add(annotation);
	}
	
	public List<Annotation> getAnnotationBuffer(){
		List<Annotation> list = this.annotationBuffer;
		this.annotationBuffer = null;
		return list;
	}
	
	@Override
	public void build(Builder builder) throws Exception {
		builder.append("package ");
		builder.append(pack);
		builder.append(";");
		
		for(Import imp : imports){
			builder.append("import ");
			if(imp.isStatic()) builder.append("static ");
			builder.append(imp.getValue());
		}
		
		mainClazz.build(builder);
		for(ClassBase clazz : subClazz){
			clazz.build(builder);
		}
	}

	@Override
	public <E extends JavaElement> E find(Function<JavaElement, Boolean> finder){
		if(finder.apply(mainClazz)) return (E) mainClazz;
		for(ClassBase clazz : subClazz){
			if(finder.apply(clazz)) return (E) clazz;
		}
		return null;
	}

	@Override
	public <E extends JavaElement> void findAll(Function<JavaElement, Boolean> finder, List<E> list){
		if(finder.apply(mainClazz)) list.add((E) mainClazz);
		mainClazz.findAll(finder, list);
		for(ClassBase clazz : subClazz){
			if(finder.apply(clazz)) list.add((E) clazz);
			clazz.findAll(finder, list);
		}
	}

	@Override
	public VisitorBag visit(JavaVisitor<?> visitor){
		if(visitor.canVisit(getClass())) return visitor.visit(this);
		VisitorBag bag = new VisitorBag();
		
		if(this.imports != null)
			for(Import imp : this.imports){
				bag.merge(imp.visit(visitor));
			}
		
		bag.merge(mainClazz.visit(visitor));
		
		if(this.subClazz != null)
			for(ClassBase clazz : this.subClazz){
				bag.merge(clazz.visit(visitor));
			}
		
		return bag;
	}
	
}
