package dev.peerat.parser.java;

import java.io.BufferedReader;

import dev.peerat.parser.Token;
import dev.peerat.parser.TokenType;
import dev.peerat.parser.Tokenizer;

public class JavaTokenizer extends Tokenizer{
	
	@Override
	public void parse(BufferedReader reader) throws Exception{
		int lineNumber = 0;
		String line;
		boolean longCommentary = false;
		while((line = reader.readLine()) != null){
			lineNumber++;
			
			for(int i = 0; i < line.length(); i++){
				char c = line.charAt(i);
				if(longCommentary){
					if(c == '*' && (i < line.length()-1 && line.charAt(i+1) == '/')){
						longCommentary = false;
						i++;
					}
//					System.out.print(c);
					continue;
				}
				Token token;
				if(isNameValid(c)){
					String value = "";
					int j = i;
					for(; j < line.length(); j++){
						c = line.charAt(j);
						if(isNameValid(c)) value+=c;
						else break;
					}
					token = new Token(lineNumber, i+1, value, TokenType.NAME);
					i = j-1;
				}else if(Character.isWhitespace(c)) continue;
				else{
					if(c == '"'){
						String value = "\"";
						int j = i+1;
						for(; j < line.length(); j++){
							c = line.charAt(j);
							if(c == '"'){
								value+=c;
								j++;
								break;
							}else if(c == '\\'){
								value+=c+line.charAt(++j);
							}else{
								value+=c;
							}
						}
						token = new Token(lineNumber, i+1, value, TokenType.STRING);
						i = j-1;
					}else if(c == '\''){
						String value = "'";
						int j = i+1;
						for(; j < line.length(); j++){
							c = line.charAt(j);
							if(c == '\''){
								value+=c;
								j++;
								break;
							}else if(c == '\\'){
								value+=c+line.charAt(++j);
							}else{
								value+=c;
							}
						}
						token = new Token(lineNumber, i+1, value, TokenType.CHAR);
						i = j-1;
					}else{
						if(c == '/' && (i < line.length()-1 && line.charAt(i+1) == '/')) break;
						if(c == '/' && (i < line.length()-1 && line.charAt(i+1) == '*')){
							longCommentary = true;
							continue;
						}
						token = new Token(lineNumber, i+1, ""+c, TokenType.DELIMITER);
					}
				}
				getTokens().add(token);
			}
		}
	}
	
	@Override
	public void parse(String line){
		System.out.println("tokenizer "+line);
		int lineNumber = 1;
		boolean longCommentary = false;
		for(int i = 0; i < line.length(); i++){
			char c = line.charAt(i);
			if(longCommentary){
				if(c == '*' && (i < line.length()-1 && line.charAt(i+1) == '/')){
					longCommentary = false;
					i++;
				}
//				System.out.print(c);
				continue;
			}
			Token token;
			if(isNameValid(c)){
				String value = "";
				int j = i;
				for(; j < line.length(); j++){
					c = line.charAt(j);
					if(isNameValid(c)) value+=c;
					else break;
				}
				token = new Token(lineNumber, i+1, value, TokenType.NAME);
				i = j-1;
			}else if(Character.isWhitespace(c)) continue;
			else{
				if(c == '"'){
					String value = "\"";
					int j = i+1;
					for(; j < line.length(); j++){
						c = line.charAt(j);
						if(c == '"'){
							value+=c;
							j++;
							break;
						}else if(c == '\\'){
							value+=c+line.charAt(++j);
						}else{
							value+=c;
						}
					}
					token = new Token(lineNumber, i+1, value, TokenType.STRING);
					i = j-1;
					System.out.println(token);
				}else if(c == '\''){
					String value = "'";
					int j = i+1;
					for(; j < line.length(); j++){
						c = line.charAt(j);
						if(c == '\''){
							value+=c;
							j++;
							break;
						}else if(c == '\\'){
							value+=c+line.charAt(++j);
						}else{
							value+=c;
						}
					}
					token = new Token(lineNumber, i+1, value, TokenType.CHAR);
					i = j-1;
					System.out.println(token);
				}else{
					if(c == '/' && (i < line.length()-1 && line.charAt(i+1) == '/')) break;
					if(c == '/' && (i < line.length()-1 && line.charAt(i+1) == '*')){
						longCommentary = true;
						continue;
					}
					token = new Token(lineNumber, i+1, ""+c, TokenType.DELIMITER);
				}
			}
			getTokens().add(token);
		}
	}
	
	private boolean isNameValid(char c) {
		return Character.isAlphabetic(c) || Character.isDigit(c) || c == '_' || c == '$';
	}

}
