package dev.peerat.parser.java.tree;

import java.util.LinkedHashMap;
import java.util.Map;

import dev.peerat.parser.Token;
import dev.peerat.parser.TokenType;
import dev.peerat.parser.java.Annotation;
import dev.peerat.parser.java.Annotation.AnnotableBuffer;
import dev.peerat.parser.java.JavaElement;
import dev.peerat.parser.java.value.Value;
import dev.peerat.parser.state.RedirectStateTree;
import dev.peerat.parser.state.StateTree;
import dev.peerat.parser.tree.SyntaxTree;
import dev.peerat.parser.tree.SyntaxTreeRegistery;

public class AnnotationTree extends SyntaxTree<JavaElement> {

	public AnnotationTree(SyntaxTreeRegistery<JavaElement> registery) {
		super(registery);
		this.current = new StateTree<>();
	}

	@Override
	public void init() {
		StateTree<JavaElement> value = registery.getNode(ValueTree.class);
		// ANNOTATION
		StateTree<JavaElement> annotation = this.current;
		StateTree<JavaElement> annotation_name = annotation
				.then((validator) -> validator.validate((token) -> token.getValue().equals("@"))
						&& !validator.validate((token) -> token.getValue().equals("interface")))
				.then((validator) -> validator.validate((token) -> token.getType().equals(TokenType.NAME),
						(bag, token) -> {
							Token name = bag.get("name");
							if (name == null)
								bag.set("name", token);
							else
								bag.set("name", name.concat(token));
						}));
		annotation_name
				.then((validator) -> validator.validate((token) -> token.getValue().equals("."), (bag, token) -> {
					Token name = bag.get("name");
					if (name == null)
						bag.set("name", token);
					else
						bag.set("name", name.concat(token));
				})).then(annotation_name);
		annotation_name.end((parent, bag) -> {
			Annotation result = new Annotation(bag.get("name"));
			bag.set(result);
			if (parent instanceof AnnotableBuffer)
				((AnnotableBuffer) parent).addAnnotationBuffer(result);
			return null;
		});
		StateTree<JavaElement> annotation_begin = annotation_name
				.then((validator) -> validator.validate((token) -> token.getValue().equals("(")));
		StateTree<JavaElement> annotation_end = annotation_begin
				.then((validator) -> validator.validate((token) -> token.getValue().equals(")")));
		annotation_end.end((parent, bag) -> {
			Annotation result = new Annotation(bag.get("name"), bag.get("values"));
			bag.set(result);
			if (parent instanceof AnnotableBuffer)
				((AnnotableBuffer) parent).addAnnotationBuffer(result);
			return null;
		});
		StateTree<JavaElement> annotation_var = annotation_begin.then((validator) -> validator
				.validate((token) -> token.getType().equals(TokenType.NAME), (bag, token) -> bag.set("index", token)));
		StateTree<JavaElement> annotation_value = annotation_var
				.then((validator) -> validator.validate((token) -> token.getValue().equals("=")))
				.then(new RedirectStateTree<>(value, (global, local) -> {
					Map<Token, Value> map = global.get("values");
					if (map == null) {
						map = new LinkedHashMap<>();
						global.set("values", map);
					}
					map.put(global.get("index"), local.get());
				}));
		annotation_value.then((validator) -> validator.validate((token) -> token.getValue().equals(",")))
				.then(annotation_var);
		annotation_value.then(annotation_end);
		annotation_begin.then(new RedirectStateTree<>(value, (global, local) -> {
			Map<Token, Value> map = global.get("values");
			if (map == null) {
				map = new LinkedHashMap<>();
				global.set("values", map);
			}
			map.put(null, local.get());
		})).then(annotation_end);
	}

}
