package dev.peerat.parser.java.tree;

import java.lang.reflect.Modifier;

import dev.peerat.parser.java.JavaElement;
import dev.peerat.parser.state.StateTree;
import dev.peerat.parser.tree.SyntaxTree;
import dev.peerat.parser.tree.SyntaxTreeRegistery;

public class ModifierTree extends SyntaxTree<JavaElement> {

	public ModifierTree(SyntaxTreeRegistery<JavaElement> registery) {
		super(registery);
		this.current = new StateTree<>();
	}

	@Override
	public void init() {
		this.current.then((validator) -> {
			boolean mod = false;
			while (validator.validate((token) -> getModifier(token.getValue()) > 0, (bag, token) -> {
				int m = getModifier(token.getValue());
				Integer current = bag.get();
				bag.set(current == null ? m : current + m);
			}))
				mod = true;
			return mod;
		}).end();
	}

	private int getModifier(String modifier) {
		switch (modifier) {
		case "public":
			return Modifier.PUBLIC;
		case "private":
			return Modifier.PRIVATE;
		case "protected":
			return Modifier.PROTECTED;
		case "static":
			return Modifier.STATIC;
		case "final":
			return Modifier.FINAL;
		case "synchronized":
			return Modifier.SYNCHRONIZED;
		case "volatile":
			return Modifier.VOLATILE;
		case "transient":
			return Modifier.TRANSIENT;
		case "native":
			return Modifier.NATIVE;
		case "abstract":
			return Modifier.ABSTRACT;
		case "strictfp":
			return Modifier.STRICT;
		case "default":
			return Modifier.STRICT;
		default:
			break;
		}
		return 0;
	}
}
