package dev.peerat.parser.java.tree;

import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.function.BiFunction;

import dev.peerat.parser.Bag;
import dev.peerat.parser.Token;
import dev.peerat.parser.TokenType;
import dev.peerat.parser.java.Annotation;
import dev.peerat.parser.java.JavaElement;
import dev.peerat.parser.java.Variable;
import dev.peerat.parser.java.Annotation.AnnotableBuffer;
import dev.peerat.parser.java.Variable.VariableContainer;
import dev.peerat.parser.java.value.Value;
import dev.peerat.parser.state.InitialStateTree;
import dev.peerat.parser.state.RedirectStateTree;
import dev.peerat.parser.state.StateTree;
import dev.peerat.parser.tree.SyntaxTree;
import dev.peerat.parser.tree.SyntaxTreeRegistery;

public class VariableTree extends SyntaxTree<JavaElement> {

	public VariableTree(SyntaxTreeRegistery<JavaElement> registery) {
		super(registery);
		this.current = new InitialStateTree<>();
	}

	@Override
	public void init() {
		StateTree<JavaElement> modifier = registery.getNode(ModifierTree.class);
		StateTree<JavaElement> type = registery.getNode(TypeTree.class);
		StateTree<JavaElement> value = registery.getNode(ValueTree.class);
		StateTree<JavaElement> annotation = registery.getNode(AnnotationTree.class);

		// VARIABLE
		BiFunction<JavaElement, Bag, JavaElement> variable_builder = (parent, bag) -> {
			Integer mod = bag.get("mod");
			Token vtype = bag.get("type");
			Map<Token, Value> map = bag.get("vars");
			for (Entry<Token, Value> vars : map.entrySet()) {
				List<Annotation> annotations = null;
				if (parent instanceof AnnotableBuffer)
					annotations = (((AnnotableBuffer) parent).getAnnotationBuffer());
				Variable result = new Variable(annotations, mod == null ? 0 : mod, vtype, vars.getKey(), false,
						vars.getValue());
				bag.set(result);
				if (parent instanceof VariableContainer)
					((VariableContainer) parent).addVariable(result);
			}
			return null;
		};

		InitialStateTree<JavaElement> variable = (InitialStateTree<JavaElement>) this.current;
		variable.multiple(annotation);
		StateTree<JavaElement> variable_mod = variable
				.then(new RedirectStateTree<>(modifier, (global, local) -> global.set("mod", local.get())));
		StateTree<JavaElement> variable_type = variable
				.then(new RedirectStateTree<>(type, (global, local) -> global.set("type", local.get())));
		variable_mod.then(variable_type);
		StateTree<JavaElement> variable_name = variable_type.then(
				(validator) -> validator.validate((token) -> token.getType().equals(TokenType.NAME), (bag, token) -> {
					Map<Token, Value> map = bag.get("vars");
					if (map == null) {
						map = new LinkedHashMap<>();
						bag.set("vars", map);
					}
					bag.set("last", token);
					map.put(token, null);
				}));
		variable_name.then((validator) -> validator.validate((token) -> token.getValue().equals(";")))
				.end(variable_builder);
		StateTree<JavaElement> variable_split = variable_name
				.then((validator) -> validator.validate((token) -> token.getValue().equals(",")));
		variable_split.then(variable_name);
		StateTree<JavaElement> variable_value = variable_name
				.then((validator) -> validator.validate((token) -> token.getValue().equals("=")))
				.then(new RedirectStateTree<>(value, (global, local) -> {
					global.<Map<Token, Value>>get("vars").put(global.get("last"), local.get());
				}));
		variable_value.then(variable_split);
		variable_value.then((validator) -> validator.validate((token) -> token.getValue().equals(";")))
				.end(variable_builder);
		variable_value.end(variable_builder);
	}

}
